import csv
import io

from lodstorage.jsonable import JSONAble
from lodstorage.lod import LOD


class CSV(LOD):
    '''
    helper for converting data in csv format to list of dicts (LoD) and vice versa
    '''

    @staticmethod
    def restoreFromCSVFile(filePath:str, headerNames:list=None, withPostfix:bool=False):
        '''
        restore LOD from given csv file

        Args:
            filePath(str): file name
            headerNames(list): Names of the headers that should be used. If None it is assumed that the header is given.
            withPostfix(bool): If False the file type is appended to given filePath. Otherwise file type MUST be given with filePath.

        Returns:
            list of dicts (LoD) containing the content of the given csv file
        '''
        if not withPostfix:
            filePath += ".csv"
        csvStr = CSV.readFile(filePath)
        lod = CSV.fromCSV(csvStr, headerNames)
        return lod

    @staticmethod
    def fromCSV(csvString:str, fields:list=None, delimiter=",",quoting=csv.QUOTE_NONNUMERIC, **kwargs):
        '''
        convert given csv string to list of dicts (LOD)

        Args:
            csvStr(str): csv string that should be converted to LOD
            headerNames(list): Names of the headers that should be used. If None it is assumed that the header is given.

        Returns:
            list of dicts (LoD) containing the content of the given csv string
        '''
        csvStream=io.StringIO(csvString)
        reader = csv.DictReader(csvStream, fieldnames=fields, delimiter=delimiter, quoting=quoting, **kwargs)
        lod=list(reader)
        CSV.fixTypes(lod)
        return lod

    @staticmethod
    def storeToCSVFile(lod:list, filePath:str, withPostfix:bool=False):
        '''
            converts the given lod to CSV file.

            Args:
                lod(list): lod that should be converted to csv file
                filePath(str): file name the csv should be stored to
                withPostfix(bool): If False the file type is appended to given filePath. Otherwise file type MUST be given with filePath.
            Returns:
                csv string of the given lod
        '''
        if not withPostfix:
            filePath += ".csv"
        csvStr=CSV.toCSV(lod)
        CSV.writeFile(csvStr, filePath)

    @staticmethod
    def toCSV(lod:list, includeFields:list=None, excludeFields:list=None, delimiter=",",quoting=csv.QUOTE_NONNUMERIC, **kwargs):
        '''
        converts the given lod to CSV string.
        For details about the csv dialect parameters see https://docs.python.org/3/library/csv.html#csv-fmt-params

        Args:
            lod(list): lod that should be converted to csv string
            includeFields(list): list of fields that should be included in the csv (positive list)
            excludeFields(list): list of fields that should be excluded from the csv (negative list)
            kwargs: csv dialect parameters
        Returns:
            csv string of the given lod
        '''
        if lod is None:
            return ''
        if isinstance(lod[0], JSONAble):
            lod=[vars(d) for d in lod]
        if excludeFields is not None:
            lod=LOD.filterFields(lod, excludeFields)
        if includeFields is None:
            fields = LOD.getFields(lod)
        else:
            fields=includeFields
            lod=LOD.filterFields(lod, includeFields, reverse=True)
        csvStream = io.StringIO()
        dict_writer = csv.DictWriter(csvStream, fieldnames=fields, delimiter=delimiter, quoting=quoting, **kwargs)
        dict_writer.writeheader()
        dict_writer.writerows(lod)
        csvString = csvStream.getvalue()
        return csvString

    @staticmethod
    def readFile(filename: str) -> str:
        """
        Reads the given filename and returns it as string
        Args:
            filename: Name of the file that should be returned as string

        Returns:
            Content of the file as string
        """
        with open(filename, 'r') as file:
            content = file.read()
        return content

    @staticmethod
    def writeFile(content:str, filename: str) -> str:
        """
        Write the given str to the given filename
        Args:
            content(str): string that should be written into the file
            filename: Name of the file the given str should be written to
        Returns:
            Nothing
        """
        with open(filename, 'w') as file:
            file.write(content)

    @staticmethod
    def fixTypes(lod:list):
        """
        fixes the types of the given LoD.

        """
        for record in lod:
            for key, value in record.items():
                # fix empty csv value: "cell1,,cell3" converts the second value to empty string instead of None
                if value == '':
                    record[key] = None




'''
Created on 2022-03-06

@author: wf
'''

class Version(object):
    '''
    Version handling for ConferenceCorpus
    '''
    name="pyLodStorage"
    version='0.1.20'
    date = '2020-09-10'
    updated = '2022-03-14'

        


'''
Created on 2020-08-29

@author: wf
'''
from enum import Enum
from pathlib import Path
import os
class StoreMode(Enum):
    '''
    possible supported storage modes
    '''
    JSONPICKLE = 1      # JSON Pickle
    JSON = 2
    SQL = 3
    SPARQL = 4
    YAML = 5    
    
class StorageConfig(object):
    '''
    a storage configuration
    '''
           
    def getCachePath(self,ensureExists=True)->str:
        '''
        get the path to the default cache
        
        Args:
            name(str): the name of the cache to use
        '''
        
        cachedir=f"{self.cacheRootDir}/.{self.cacheDirName}"
        
        if ensureExists:
            if not os.path.exists(cachedir):
                os.makedirs(cachedir)
        return cachedir

    def __init__(self, mode=StoreMode.SQL,cacheRootDir:str=None,cacheDirName:str="lodstorage",cacheFile=None,withShowProgress=True,profile=True,debug=False,errorDebug=True):
        '''
        Constructor
        
        Args:
            mode(StoreMode): the storage mode e.g. sql
            cacheRootDir(str): the cache root directory to use - if None the home directory will be used
            cacheFile(string): the common cacheFile to use (if any)
            withShowProgress(boolean): True if progress should be shown
            profile(boolean): True if timing / profiling information should be shown
            debug(boolean): True if debugging information should be shown
            errorDebug(boolean): True if debug info should be provided on errors (should not be used for production since it might reveal data)
        '''
        if cacheRootDir is None:
            home = str(Path.home())
            self.cacheRootDir=f"{home}"
        else:
            self.cacheRootDir=cacheRootDir
        self.cacheDirName=cacheDirName
        self.mode=mode
        self.cacheFile=cacheFile
        self.profile=profile
        self.withShowProgress=withShowProgress
        self.debug=debug
        self.errorDebug=errorDebug
        
    @staticmethod
    def getDefault(debug=False):
        return StorageConfig.getSQL(debug)    
        
    @staticmethod
    def getSQL(debug=False):
        config=StorageConfig(mode=StoreMode.SQL,debug=debug)
        config.tableName=None
        return config
    
    @staticmethod
    def getJSON(debug=False):
        config=StorageConfig(mode=StoreMode.JSON,debug=debug)
        return config
    
    @staticmethod
    def getJsonPickle(debug=False):
        config=StorageConfig(mode=StoreMode.JSONPICKLE,debug=debug)
        return config
    
    @staticmethod
    def getSPARQL(prefix,endpoint,host,debug=False):
        config=StorageConfig(mode=StoreMode.SPARQL,debug=debug)
        config.prefix=prefix
        config.host=host
        config.endpoint=endpoint
        return config
    
    @staticmethod
    def getYaml(debug=False):
        config=StorageConfig(mode=StoreMode.YAML,debug=debug)
        return config
    
    